@@ -3,24 +3,28 @@
 from __future__ import annotations
 
 import operator
-from typing import TYPE_CHECKING, Any, Callable, Sequence
+from typing import TYPE_CHECKING, Any, Callable, Hashable, Mapping, Sequence
 
 import boost_histogram as bh
-import dask.array as da
-import dask.bag as db
-from dask.bag.core import empty_safe_aggregate, partition_all
-from dask.base import DaskMethodsMixin, is_dask_collection, tokenize
+import numpy as np
+from dask.array.core import Array as DaskArray
+from dask.array.core import asarray
+from dask.bag.core import empty_safe_aggregate
+from dask.base import DaskMethodsMixin, dont_optimize, is_dask_collection, tokenize
+from dask.blockwise import fuse_roots, optimize_blockwise
+from dask.context import globalmethod
+from dask.core import flatten
 from dask.dataframe.core import partitionwise_graph as partitionwise
+from dask.delayed import Delayed
 from dask.highlevelgraph import HighLevelGraph
 from dask.threaded import get as tget
 from dask.utils import is_dataframe_like, key_split
+from tlz import partition_all
 
 if TYPE_CHECKING:
-    import numpy as np
+    from numpy.typing import NDArray
 
-    from .typing import DaskCollection
-else:
-    DaskCollection = object
+    from dask_histogram.typing import DaskCollection
 
 __all__ = (
     "AggHistogram",
@@ -30,7 +34,7 @@ __all__ = (
 )
 
 
-def clone(histref: bh.Histogram = None) -> bh.Histogram:
+def clone(histref: bh.Histogram | None = None) -> bh.Histogram:
     """Create a Histogram object based on another.
 
     The axes and storage of the `histref` will be used to create a new
@@ -52,71 +56,172 @@ def clone(histref: bh.Histogram = None) -> bh.Histogram:
     return bh.Histogram(*histref.axes, storage=histref._storage_type())
 
 
-def _blocked_sa_w(
+def _blocked_sa(
+    data: Any,
+    *,
+    histref: bh.Histogram | None = None,
+) -> bh.Histogram:
+    """Blocked calculation; single argument; unweighted; no sample."""
+    if data.ndim == 1:
+        return clone(histref).fill(data)
+    elif data.ndim == 2:
+        return clone(histref).fill(*(data.T))
+    else:
+        raise ValueError("Data must be one or two dimensional.")
+
+
+def _blocked_sa_s(
+    data: Any,
     sample: Any,
+    *,
+    histref: bh.Histogram | None = None,
+) -> bh.Histogram:
+    """Blocked calculation; single argument; unweighted; with sample."""
+    if data.ndim == 1:
+        return clone(histref).fill(data, sample=sample)
+    elif data.ndim == 2:
+        return clone(histref).fill(*(data.T), sample=sample)
+    else:
+        raise ValueError("Data must be one or two dimensional.")
+
+
+def _blocked_sa_w(
+    data: Any,
     weights: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
-    """Blocked calculation; single argument; weighted."""
-    if sample.ndim == 1:
-        return clone(histref).fill(sample, weight=weights)
-    elif sample.ndim == 2:
-        return clone(histref).fill(*(sample.T), weight=weights)
+    """Blocked calculation; single argument; weighted; no sample."""
+    if data.ndim == 1:
+        return clone(histref).fill(data, weight=weights)
+    elif data.ndim == 2:
+        return clone(histref).fill(*(data.T), weight=weights)
     else:
         raise ValueError("Data must be one or two dimensional.")
 
 
-def _blocked_sa(
+def _blocked_sa_w_s(
+    data: Any,
+    weights: Any,
     sample: Any,
     *,
-    histref: bh.Histogram = None,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
-    """Blocked calculation; single argument; unweighted."""
-    if sample.ndim == 1:
-        return clone(histref).fill(sample, weight=None)
-    elif sample.ndim == 2:
-        return clone(histref).fill(*(sample.T), weight=None)
+    """Blocked calculation; single argument; weighted; with sample."""
+    if data.ndim == 1:
+        return clone(histref).fill(data, weight=weights, sample=sample)
+    elif data.ndim == 2:
+        return clone(histref).fill(*(data.T), weight=weights, sample=sample)
     else:
         raise ValueError("Data must be one or two dimensional.")
 
 
-def _blocked_ma_w(
-    *sample: Any,
-    histref: bh.Histogram = None,
-) -> bh.Histogram:
-    """Blocked calculation; multiargument; unweighted."""
-    weights = sample[-1]
-    sample = sample[:-1]
-    return clone(histref).fill(*sample, weight=weights)
-
-
 def _blocked_ma(
-    *sample: Any,
-    histref: bh.Histogram = None,
+    *data: Any,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
-    """Blocked calculation; multiargument; unweighted."""
-    return clone(histref).fill(*sample, weight=None)
+    """Blocked calculation; multiargument; unweighted; no sample."""
+    return clone(histref).fill(*data)
 
 
-def _blocked_df_w(
-    sample: Any,
-    weights: Any,
-    *,
-    histref: bh.Histogram = None,
+def _blocked_ma_s(
+    *data: Any,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
-    """Blocked calculation; single argument; weighted."""
-    return clone(histref).fill(*(sample[c] for c in sample.columns), weight=weights)
+    """Blocked calculation; multiargument; unweighted; with sample."""
+    sample = data[-1]
+    data = data[:-1]
+    return clone(histref).fill(*data, sample=sample)
+
+
+def _blocked_ma_w(
+    *data: Any,
+    histref: bh.Histogram | None = None,
+) -> bh.Histogram:
+    """Blocked calculation; multiargument; weighted; no sample."""
+    weights = data[-1]
+    data = data[:-1]
+    return clone(histref).fill(*data, weight=weights)
+
+
+def _blocked_ma_w_s(
+    *data: Any,
+    histref: bh.Histogram | None = None,
+) -> bh.Histogram:
+    """Blocked calculation; multiargument; weighted; with sample."""
+    weights = data[-2]
+    sample = data[-1]
+    data = data[:-2]
+    return clone(histref).fill(*data, weight=weights, sample=sample)
 
 
 def _blocked_df(
-    sample: Any,
-    histref: bh.Histogram = None,
+    data: Any,
+    *,
+    histref: bh.Histogram | None = None,
 ) -> bh.Histogram:
-    return clone(histref).fill(*(sample[c] for c in sample.columns), weight=None)
+    return clone(histref).fill(*(data[c] for c in data.columns), weight=None)
 
 
-class AggHistogram(db.Item):
+def _blocked_df_s(
+    data: Any,
+    sample: Any,
+    *,
+    histref: bh.Histogram | None = None,
+) -> bh.Histogram:
+    return clone(histref).fill(*(data[c] for c in data.columns), sample=sample)
+
+
+def _blocked_df_w(
+    data: Any,
+    weights: Any,
+    *,
+    histref: bh.Histogram | None = None,
+) -> bh.Histogram:
+    """Blocked calculation; single argument; weighted; no sample."""
+    return clone(histref).fill(*(data[c] for c in data.columns), weight=weights)
+
+
+def _blocked_df_w_s(
+    data: Any,
+    weights: Any,
+    sample: Any,
+    *,
+    histref: bh.Histogram | None = None,
+) -> bh.Histogram:
+    """Blocked calculation; single argument; weighted; with sample."""
+    return clone(histref).fill(
+        *(data[c] for c in data.columns), weight=weights, sample=sample
+    )
+
+
+def _blocked_dak(data: Any, *, histref: bh.Histogram | None = None) -> bh.Histogram:
+    return clone(histref).fill(data)
+
+
+def _blocked_dak_ma(*data: Any, histref: bh.Histogram | None = None) -> bh.Histogram:
+    return clone(histref).fill(*data)
+
+
+def optimize(
+    dsk: Mapping,
+    keys: Hashable | list[Hashable] | set[Hashable],
+    **kwargs: Any,
+) -> Mapping:
+    if not isinstance(keys, (list, set)):
+        keys = [keys]
+    keys = list(flatten(keys))
+
+    if not isinstance(dsk, HighLevelGraph):
+        dsk = HighLevelGraph.from_collections(id(dsk), dsk, dependencies=())
+
+    dsk = optimize_blockwise(dsk, keys=keys)
+    dsk = fuse_roots(dsk, keys=keys)  # type: ignore
+    dsk = dsk.cull(set(keys))  # type: ignore
+    return dsk
+
+
+class AggHistogram(DaskMethodsMixin):
     """Aggregated Histogram collection.
 
     The class constructor is typically used internally;
@@ -129,19 +234,76 @@ class AggHistogram(db.Item):
 
     """
 
-    def __init__(self, dsk: HighLevelGraph, key: str, histref: bh.Histogram) -> None:
-        self.dask: HighLevelGraph = dsk
-        self.key: str = key
-        self._histref: bh.Histogram = histref
+    def __init__(
+        self,
+        dsk: HighLevelGraph,
+        name: str,
+        histref: bh.Histogram,
+        layer: Any | None = None,
+    ) -> None:
+        self._dask: HighLevelGraph = dsk
+        self._name: str = name
+        self._meta: bh.Histogram = histref
+
+        # NOTE: Layer only used by `Item.from_delayed`, to handle
+        # Delayed objects created by other collections. e.g.:
+        # Item.from_delayed(da.ones(1).to_delayed()[0]) See
+        # Delayed.__init__
+        self._layer = layer or name
+        if isinstance(dsk, HighLevelGraph) and self._layer not in dsk.layers:
+            raise ValueError(
+                f"Layer {self._layer} not in the HighLevelGraph's layers: {list(dsk.layers)}"
+            )
+
+    def __dask_graph__(self) -> HighLevelGraph:
+        return self._dask
+
+    def __dask_keys__(self) -> list[str]:
+        return [self.name]
+
+    def __dask_layers__(self) -> tuple[str, ...]:
+        if isinstance(self._dask, HighLevelGraph) and len(self._dask.layers) == 1:
+            return tuple(self._dask.layers)
+        return (self.name,)
+
+    def __dask_tokenize__(self) -> str:
+        return self.name
+
+    def __dask_postcompute__(self) -> Any:
+        return _finalize_agg_histogram, ()
+
+    def __dask_postpersist__(self) -> Any:
+        return self._rebuild, ()
+
+    __dask_optimize__ = globalmethod(
+        optimize, key="histogram_optimize", falsey=dont_optimize
+    )
+
+    __dask_scheduler__ = staticmethod(tget)
+
+    def _rebuild(
+        self,
+        dsk: HighLevelGraph,
+        *,
+        rename: Mapping[str, str] | None = None,
+    ) -> Any:
+        name = self._name
+        if rename:
+            name = rename.get(name, name)
+        return type(self)(dsk, name, self.histref)
 
     @property
     def name(self) -> str:
-        return self.key
+        return self._name
+
+    @property
+    def dask(self) -> HighLevelGraph:
+        return self._dask
 
     @property
     def histref(self) -> bh.Histogram:
         """Empty reference boost-histogram object."""
-        return self._histref
+        return self._meta
 
     @property
     def _storage_type(self) -> type[bh.storage.Storage]:
@@ -173,21 +335,13 @@ class AggHistogram(db.Item):
         )
 
     __repr__ = __str__
-    __dask_scheduler__ = staticmethod(tget)
 
-    @property
-    def _args(self) -> tuple[HighLevelGraph, str, bh.Histogram]:
-        return (self.dask, self.key, self.histref)
-
-    def __getstate__(self) -> tuple[HighLevelGraph, str, bh.Histogram]:
-        return self._args
-
-    def __setstate__(self, state: tuple[HighLevelGraph, str, bh.Histogram]) -> None:
-        self.dask, self.key, self._histref = state
+    def __reduce__(self):
+        return (AggHistogram, (self._dask, self._name, self._meta))
 
     def to_dask_array(
         self, flow: bool = False, dd: bool = False
-    ) -> tuple[da.Array, ...] | tuple[da.Array, tuple[da.Array, ...]]:
+    ) -> tuple[DaskArray, ...] | tuple[DaskArray, list[DaskArray]]:
         """Convert histogram object to dask.array form.
 
         Parameters
@@ -216,7 +370,26 @@ class AggHistogram(db.Item):
         """
         return self.compute()
 
-    def __array__(self) -> np.ndarray:
+    def to_delayed(self, optimize_graph: bool = True) -> Delayed:
+        keys = self.__dask_keys__()
+        graph = self.__dask_graph__()
+        layer = self.__dask_layers__()[0]
+        if optimize_graph:
+            graph = self.__dask_optimize__(graph, keys)
+            layer = f"delayed-{self.name}"
+            graph = HighLevelGraph.from_collections(layer, graph, dependencies=())
+        return Delayed(keys[0], graph, layer=layer)
+
+    def values(self, flow: bool = False) -> NDArray[Any]:
+        return self.to_boost().values(flow=flow)
+
+    def variances(self, flow: bool = False) -> NDArray[Any] | None:
+        return self.to_boost().variances(flow=flow)
+
+    def counts(self, flow: bool = False) -> NDArray[Any]:
+        return self.to_boost().counts(flow=flow)
+
+    def __array__(self) -> NDArray[Any]:
         return self.compute().__array__()
 
     def __iadd__(self, other) -> AggHistogram:
@@ -260,6 +433,10 @@ def _finalize_partitioned_histogram(results: Any) -> Any:
     return results
 
 
+def _finalize_agg_histogram(results: Any) -> Any:
+    return results[0]
+
+
 class PartitionedHistogram(DaskMethodsMixin):
     """Partitioned Histogram collection.
 
@@ -275,16 +452,24 @@ class PartitionedHistogram(DaskMethodsMixin):
     """
 
     def __init__(
-        self, dsk: HighLevelGraph, key: str, npartitions: int, histref: bh.Histogram
+        self, dsk: HighLevelGraph, name: str, npartitions: int, histref: bh.Histogram
     ) -> None:
-        self.dask: HighLevelGraph = dsk
-        self.key: str = key
-        self.npartitions: int = npartitions
-        self._histref: bh.Histogram = histref
+        self._dask: HighLevelGraph = dsk
+        self._name: str = name
+        self._npartitions: int = npartitions
+        self._meta: bh.Histogram = histref
 
     @property
     def name(self) -> str:
-        return self.key
+        return self._name
+
+    @property
+    def dask(self) -> HighLevelGraph:
+        return self._dask
+
+    @property
+    def npartitions(self) -> int:
+        return self._npartitions
 
     def __dask_graph__(self) -> HighLevelGraph:
         return self.dask
@@ -307,6 +492,12 @@ class PartitionedHistogram(DaskMethodsMixin):
             name = rename.get(name, name)
         return type(self)(dsk, name, self.npartitions, self.histref)
 
+    __dask_optimize__ = globalmethod(
+        optimize, key="histogram_optimize", falsey=dont_optimize
+    )
+
+    __dask_scheduler__ = staticmethod(tget)
+
     def __str__(self) -> str:
         return "dask_histogram.PartitionedHistogram,<%s, npartitions=%d>" % (
             key_split(self.name),
@@ -314,31 +505,42 @@ class PartitionedHistogram(DaskMethodsMixin):
         )
 
     __repr__ = __str__
-    __dask_scheduler__ = staticmethod(tget)
 
-    @property
-    def _args(self) -> tuple[HighLevelGraph, str, int, bh.Histogram]:
-        return (self.dask, self.key, self.npartititions, self.histref)
-
-    def __getstate__(self) -> tuple[HighLevelGraph, str, int, bh.Histogram]:
-        return self._args
-
-    def __setstate__(
-        self, state: tuple[HighLevelGraph, str, int, bh.Histogram]
-    ) -> None:
-        self.dask, self.key, self.npartitions, self._histref = state
+    def __reduce__(self):
+        return (
+            PartitionedHistogram,
+            (
+                self._dask,
+                self._name,
+                self._npartitions,
+                self._meta,
+            ),
+        )
 
     @property
     def histref(self) -> bh.Histogram:
         """boost_histogram.Histogram: reference histogram."""
-        return self._histref
+        return self._meta
 
-    def to_agg(self, split_every: int = None) -> AggHistogram:
+    def collapse(self, split_every: int | None = None) -> AggHistogram:
         """Translate into a reduced aggregated histogram."""
         return _reduction(self, split_every=split_every)
 
+    def to_delayed(self, optimize_graph: bool = True) -> list[Delayed]:
+        keys = self.__dask_keys__()
+        graph = self.__dask_graph__()
+        layer = self.__dask_layers__()[0]
+        if optimize_graph:
+            graph = self.__dask_optimize__(graph, keys)
+            layer = f"delayed-{self.name}"
+            graph = HighLevelGraph.from_collections(layer, graph, dependencies=())
+        return [Delayed(k, graph, layer=layer) for k in keys]
 
-def _reduction(ph: PartitionedHistogram, split_every: int = None) -> AggHistogram:
+
+def _reduction(
+    ph: PartitionedHistogram,
+    split_every: int | None = None,
+) -> AggHistogram:
     if split_every is None:
         split_every = 4
     if split_every is False:
@@ -376,72 +578,125 @@ def _reduction(ph: PartitionedHistogram, split_every: int = None) -> AggHistogra
 
 def _dependencies(
     *args: DaskCollection,
-    weights: DaskCollection = None,
+    weights: DaskCollection | None = None,
+    sample: DaskCollection | None = None,
 ) -> tuple[DaskCollection, ...]:
-    if weights is not None:
+    if weights is not None and sample is None:
         return (*args, weights)
+    elif weights is None and sample is not None:
+        return (*args, sample)
+    elif weights is not None and sample is not None:
+        return (*args, weights, sample)
     return args
 
 
-def _weight_check(*data: DaskCollection, weights: DaskCollection = None) -> int:
-    if weights is None:
+def _weight_sample_check(
+    *data: DaskCollection,
+    weights: DaskCollection | None = None,
+    sample: DaskCollection | None = None,
+) -> int:
+    if weights is None and sample is None:
         return 0
-    if weights.ndim != 1:
-        raise ValueError("weights must be one dimensional.")
-    if data[0].npartitions != weights.npartitions:
-        raise ValueError("weights must have as many partitions as the data.")
+    if weights is not None:
+        if weights.ndim != 1:
+            raise ValueError("weights must be one dimensional.")
+        if data[0].npartitions != weights.npartitions:
+            raise ValueError("weights must have as many partitions as the data.")
+    if sample is not None:
+        if sample.ndim != 1:
+            raise ValueError("sample must be one dimensional.")
+        if data[0].npartitions != sample.npartitions:
+            raise ValueError("sample must have as many partitions as the data.")
     return 0
 
 
 def _partitioned_histogram(
     *data: DaskCollection,
     histref: bh.Histogram,
-    weights: DaskCollection = None,
-    split_every: int = None,
-) -> AggHistogram:
-    name = f"hist-on-block-{tokenize(data, histref, weights)}"
+    weights: DaskCollection | None = None,
+    sample: DaskCollection | None = None,
+    split_every: int | None = None,
+) -> PartitionedHistogram:
+    name = f"hist-on-block-{tokenize(data, histref, weights, sample, split_every)}"
     data_is_df = is_dataframe_like(data[0])
-    _weight_check(*data, weights=weights)
-    if len(data) == 1 and not data_is_df:
+    data_is_dak = is_awkward_like(data[0])
+    _weight_sample_check(*data, weights=weights)
+
+    # Single awkward array object.
+    if len(data) == 1 and data_is_dak:
+        from dask_awkward.core import partitionwise_layer as dak_pwl
+
         x = data[0]
-        if weights is not None:
+        if weights is not None and sample is not None:
+            raise NotImplementedError()
+        elif weights is not None and sample is None:
+            raise NotImplementedError()
+        elif weights is None and sample is not None:
+            raise NotImplementedError()
+        else:
+            g = dak_pwl(_blocked_dak, name, x, histref=histref)
+
+    # Single object, not a dataframe
+    elif len(data) == 1 and not data_is_df:
+        x = data[0]
+        if weights is not None and sample is not None:
+            g = partitionwise(
+                _blocked_sa_w_s, name, x, weights, sample, histref=histref
+            )
+        elif weights is not None and sample is None:
             g = partitionwise(_blocked_sa_w, name, x, weights, histref=histref)
+        elif weights is None and sample is not None:
+            g = partitionwise(_blocked_sa_s, name, x, sample, histref=histref)
         else:
             g = partitionwise(_blocked_sa, name, x, histref=histref)
+
+    # Single object, is a dataframe
     elif len(data) == 1 and data_is_df:
         x = data[0]
-        if weights is not None:
+        if weights is not None and sample is not None:
+            g = partitionwise(
+                _blocked_df_w_s, name, x, weights, sample, histref=histref
+            )
+        elif weights is not None and sample is None:
             g = partitionwise(_blocked_df_w, name, x, weights, histref=histref)
+        elif weights is None and sample is not None:
+            g = partitionwise(_blocked_df_s, name, x, sample, histref=histref)
         else:
             g = partitionwise(_blocked_df, name, x, histref=histref)
+
+    # Multiple objects
     else:
-        if weights is not None:
+
+        # Awkward array collection detected as first argument
+        if data_is_dak:
+            from dask_awkward.core import partitionwise_layer as dak_pwl
+
+            if weights is None and sample is None:
+                g = dak_pwl(_blocked_dak_ma, name, *data, histref=histref)
+            else:
+                raise NotImplementedError()
+        # Not an awkward array collection
+        elif weights is not None and sample is not None:
+            g = partitionwise(
+                _blocked_ma_w_s, name, *data, weights, sample, histref=histref
+            )
+        elif weights is not None and sample is None:
             g = partitionwise(_blocked_ma_w, name, *data, weights, histref=histref)
+        elif weights is None and sample is not None:
+            g = partitionwise(_blocked_ma_s, name, *data, sample, histref=histref)
         else:
             g = partitionwise(_blocked_ma, name, *data, histref=histref)
 
-    dependencies = _dependencies(*data, weights=weights)
+    dependencies = _dependencies(*data, weights=weights, sample=sample)
     hlg = HighLevelGraph.from_collections(name, g, dependencies=dependencies)
     return PartitionedHistogram(hlg, name, data[0].npartitions, histref=histref)
 
 
-def _reduced_histogram(
-    *data: DaskCollection,
-    histref: bh.Histogram,
-    weights: DaskCollection = None,
-    split_every: int = None,
-) -> AggHistogram:
-    ph = _partitioned_histogram(
-        *data, histref=histref, weights=weights, split_every=split_every
-    )
-    return ph.to_agg(split_every=split_every)
-
-
 def to_dask_array(
     agghist: AggHistogram,
     flow: bool = False,
     dd: bool = False,
-) -> tuple[DaskCollection, list[DaskCollection]] | tuple[DaskCollection, ...]:
+) -> tuple[DaskArray, ...] | tuple[DaskArray, list[DaskArray]]:
     """Convert `agghist` to a `dask.array` return style.
 
     Parameters
@@ -469,7 +724,7 @@ def to_dask_array(
     """
     name = f"to-dask-array-{tokenize(agghist)}"
     zeros = (0,) * agghist.histref.ndim
-    dsk = {(name, *zeros): (lambda x, f: x.to_numpy(flow=f)[0], agghist.key, flow)}
+    dsk = {(name, *zeros): (lambda x, f: x.to_numpy(flow=f)[0], agghist.name, flow)}
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=(agghist,))
     shape = agghist.histref.shape
     if flow:
@@ -479,16 +734,26 @@ def to_dask_array(
         bh.storage.AtomicInt64,
     )
     dt = int if int_storage else float
-    c = da.Array(graph, name=name, shape=shape, chunks=shape, dtype=dt)
+    c = DaskArray(graph, name=name, shape=shape, chunks=shape, dtype=dt)
     axes = agghist.histref.axes
-    edges = (da.asarray(ax.edges) for ax in axes)
+
+    if flow:
+        edges = [
+            asarray(np.concatenate([[-np.inf], ax.edges, [np.inf]])) for ax in axes
+        ]
+    else:
+        edges = [asarray(ax.edges) for ax in axes]
     if dd:
-        return (c, list(edges))
-    return (c, *(tuple(edges)))
+        return c, edges
+    return (c, *tuple(edges))
 
 
 class BinaryOpAgg:
-    def __init__(self, func: Callable[[Any, Any], Any], name: str = None) -> None:
+    def __init__(
+        self,
+        func: Callable[[Any, Any], Any],
+        name: str | None = None,
+    ) -> None:
         self.func = func
         self.__name__ = func.__name__ if name is None else name
 
@@ -499,14 +764,14 @@ class BinaryOpAgg:
             deps.append(a)
             k1 = a.name
         else:
-            k1 = a
+            k1 = a  # type: ignore
         if is_dask_collection(b):
             deps.append(b)
             k2 = b.name
         else:
-            k2 = b
-        k1 = a.__dask_tokenize__() if is_dask_collection(a) else a
-        k2 = b.__dask_tokenize__() if is_dask_collection(b) else b
+            k2 = b  # type: ignore
+        k1 = a.__dask_tokenize__() if is_dask_collection(a) else a  # type: ignore
+        k2 = b.__dask_tokenize__() if is_dask_collection(b) else b  # type: ignore
         llg = {name: (self.func, k1, k2)}
         g = HighLevelGraph.from_collections(name, llg, dependencies=deps)
         try:
@@ -524,11 +789,12 @@ _itruediv = BinaryOpAgg(operator.itruediv, name="div")
 
 def factory(
     *data: DaskCollection,
-    histref: bh.Histogram = None,
-    axes: Sequence[bh.axis.Axis] = None,
-    storage: bh.storage.Storage = None,
-    weights: DaskCollection = None,
-    split_every: int = None,
+    histref: bh.Histogram | None = None,
+    axes: Sequence[bh.axis.Axis] | None = None,
+    storage: bh.storage.Storage | None = None,
+    weights: DaskCollection | None = None,
+    sample: DaskCollection | None = None,
+    split_every: int | None = None,
     keep_partitioned: bool = False,
 ) -> AggHistogram | PartitionedHistogram:
     """Daskified Histogram collection factory function.
@@ -565,6 +831,10 @@ def factory(
     weights : DaskCollection, optional
         Weights associated with the `data`. The partitioning/chunking
         of the weights must be compatible with the input data.
+    sample : DaskCollection, optional
+        Provide samples if the histogram storage allows it. The
+        partitioning/chunking of the samples must be compatible with
+        the input data.
     split_every : int, optional
         How many blocks to use in each split during aggregation.
     keep_partitioned : bool, optional
@@ -631,5 +901,27 @@ def factory(
         if storage is None:
             storage = bh.storage.Double()
         histref = bh.Histogram(*axes, storage=storage)  # type: ignore
-    f = _partitioned_histogram if keep_partitioned else _reduced_histogram
-    return f(*data, histref=histref, weights=weights, split_every=split_every)
+
+    ph = _partitioned_histogram(*data, histref=histref, weights=weights, sample=sample)
+    if keep_partitioned:
+        return ph
+    return ph.collapse(split_every=split_every)
+
+
+def is_awkward_like(x: Any) -> bool:
+    """Check if an object is Awkward collection like.
+
+    Parameters
+    ----------
+    x : Any
+        The object of interest.
+
+    Returns
+    -------
+    bool
+        ``True`` if `x` is an Awkward Dask collection.
+
+    """
+    return (
+        hasattr(x, "__dask_graph__") and hasattr(x, "layout") and hasattr(x, "fields")
+    )
